@@ -3,7 +3,7 @@
     `sqlamp` --- Materialized Path for SQLAlchemy
     =============================================
 
-    :author: `Anton Gritsay <anton@angri.ru>`_
+    :author: `Anton Gritsay <anton@angri.ru>`_, http://angri.ru
     :version: %(version)s
     :license: 2-clause BSD (see LICENSE)
     :download: http://sqlamp.angri.ru/sqlamp-%(version)s.tar.gz
@@ -63,7 +63,7 @@ __all__ = [
     'PathOverflowError', 'TooManyChildrenError', 'PathTooDeepError'
 ]
 
-__version__ = (0, 4)
+__version__ = (0, 4, 1)
 __doc__ %= {'version': '.'.join(map(str, __version__))}
 
 
@@ -440,16 +440,19 @@ class MPClassManager(object):
         order_by = order_by or opts.pk_field
         for index in opts.indices:
             index.drop()
-        # resetting the path for roots:
-        opts.table.update().where(opts.parent_id_field == None) \
-                           .values({opts.path_field: ''}) \
-                           .execute()
         roots = sqlalchemy.select(
             [opts.pk_field], opts.parent_id_field == None
         ).order_by(order_by)
-        for root_node in roots.execute().fetchall():
+        update_query = opts.table.update()
+        for tree_id, root_node in enumerate(roots.execute().fetchall()):
             [node_id] = root_node
-            self.rebuild_subtree(node_id, order_by)
+            # resetting path, depth and tree_id for root node:
+            update_query.where(opts.pk_field == node_id) \
+                        .values({opts.tree_id_field: tree_id + 1,
+                                 opts.path_field: '',
+                                 opts.depth_field: 0}) \
+                        .execute()
+            self._do_rebuild_subtree(node_id, '', 0, tree_id + 1, order_by)
         for index in opts.indices:
             index.create()
 
@@ -691,7 +694,7 @@ class MPManager(object):
     return class manager :class:`MPClassManager` for class `Node` and
     `instance_node.mp` will return instance_node's
     :class:`MPInstanceManager`. See that classes for more details about
-    its public API.
+    their public API.
 
     :param table:
         instance of `sqlalchemy.Table`. A table that will be mapped to
@@ -713,7 +716,7 @@ class MPManager(object):
         This value allows to fine-tune the limits for max tree depth
         (equal to `(255 / steplen) + 1`) and max children in each node
         (`36 ** steplen`). Default `3` value sets the following limits:
-        max depth is `85` and max children number is `46656`.
+        max depth is `86` and max children number is `46656`.
     :param instance_manager_key='_mp_instance_manager':
         name for node instance's attribute to cache node's instance
         manager.
