@@ -2,6 +2,7 @@
 """
 `sqlamp` functional tests.
 """
+import random
 import unittest
 
 import sqlalchemy
@@ -102,22 +103,63 @@ class FunctionalTestCase(_testlib._BaseTestCase):
             _process_node(node)
         self.sess.commit()
 
+    def _corrupt_tree(self, including_roots):
+        root1, root2, root3 = self.sess.query(Cls).filter_by(parent=None) \
+                                                  .order_by('name')
+        # corrupting path:
+        used_pathes = set() # remember that they should be unique
+        for node in root1.mp.query_descendants():
+            while True:
+                path = ''.join(
+                    random.sample(
+                        sqlamp.ALPHABET + '!@#$%^&*', random.randint(1, 40)
+                    )
+                )
+                if not path in used_pathes:
+                    used_pathes.add(path)
+                    break
+            node.mp_path = path
+        if including_roots:
+            root1.mp_path = '[][][]'
+        # depth:
+        for node in root2.mp.query_descendants():
+            node.mp_depth = random.randint(10, 90)
+        if including_roots:
+            root2.mp_depth = 100
+        # tree_id:
+        for node in root3.mp.query_descendants():
+            node.mp_tree_id = _from = random.randint(1, 2000)
+        if including_roots:
+            root3.mp_tree_id = 42 + root3.mp_tree_id
+
+        self.sess.flush()
+        self.sess.expire_all()
+        return [root1, root2, root3]
+
     def test_rebuild_all_trees(self):
         self._fill_tree()
         query = sqlalchemy.select([tbl]).order_by(tbl.c.id)
         data_before = query.execute().fetchall()
+        self._corrupt_tree(including_roots=True)
+        # rebuilding all trees:
         Cls.mp.rebuild_all_trees()
+        # all trees should be in consistent state
+        # and be absolutely the same as before corruption.
         data_after = query.execute().fetchall()
         self.assertEqual(data_before, data_after)
 
-    def test_fixing_path_in_root_nodes(self):
+    def test_rebuild_subtree(self):
         self._fill_tree()
-        root1 = self.sess.query(Cls).filter_by(name='root1').one()
-        root1.mp_path = '#'
-        self.sess.flush()
-        Cls.mp.rebuild_all_trees()
-        self.sess.expire(root1)
-        self.assertNotEqual(root1.mp_path, '#')
+        query = sqlalchemy.select([tbl]).order_by(tbl.c.id)
+        data_before = query.execute().fetchall()
+        roots = self._corrupt_tree(including_roots=False)
+        # rebuilding each tree:
+        for root in roots:
+            Cls.mp.rebuild_subtree(root.id, tbl.c.id)
+        # all trees should be in consistent state
+        # and be absolutely the same as before corruption.
+        data_after = query.execute().fetchall()
+        self.assertEqual(data_before, data_after)
 
     def test_descendants(self):
         self._fill_tree()
